@@ -80,7 +80,6 @@ entry_get_text_stripped (GladeXML *xml, char const *name)
 typedef struct {
 	char *label;
 	char *uri;
-	char *location;
 	char *vendor_and_model;
 } LocalPrinter;
 
@@ -88,7 +87,6 @@ static void
 local_printer_free (LocalPrinter *printer)
 {
 	g_free (printer->label);
-	g_free (printer->location);
 	g_free (printer->uri);
 	g_free (printer->vendor_and_model);
 	g_free (printer);
@@ -112,8 +110,9 @@ combine_vendor_and_model (char const *vendor, char const *model)
 }
 
 static char *
-parse_network_detect (char *line)
+parse_network_detect (char const *line)
 {
+	char *raw_line = g_strdup (line);
 	char *vendor_and_model;
 	char *vendor = NULL;
 	char *model = NULL;
@@ -126,7 +125,7 @@ parse_network_detect (char *line)
 		*sep = '\0';
 	}
 
-	line = g_strstrip (line);
+	line = g_strstrip (raw_line);
 	fields = g_strsplit (line, ";", -1);
 
 	for (p = fields; *p != NULL; p++) {
@@ -149,6 +148,7 @@ parse_network_detect (char *line)
 	g_strfreev (fields);
 
 	if (!vendor || !model) {
+		g_free (raw_line);
 		g_free (vendor);
 		g_free (model);
 		return NULL;
@@ -158,6 +158,7 @@ parse_network_detect (char *line)
 
 	g_free (vendor);
 	g_free (model);
+	g_free (raw_line);
 
 	return vendor_and_model;
 }
@@ -175,21 +176,12 @@ set_selected_uri (GladeXML *xml, char const *uri)
 static char *
 get_selected_uri (GladeXML *xml)
 {
-	char *str;
-
-	str = g_object_get_data (G_OBJECT (xml), "selected_uri");
+	char *str = g_object_get_data (G_OBJECT (xml), "selected_uri");
 	g_return_val_if_fail (str != NULL, NULL);
 
 	return g_strdup (str);
 }
 
-static void
-set_selected_location (GladeXML *xml, char const *location)
-{
-	g_object_set_data_full (G_OBJECT (xml), "selected_location",
-				g_strdup (location), g_free);
-}
-
 /************************************************************************/
 /* Local or Remote  page */
 static void
@@ -454,31 +446,24 @@ local_location_next (GladeXML *xml)
 
 	use_detected = !toggle_button_is_active (xml, "local_specify_port_radio");
 
-	gcups_driver_selector_set_vendor_and_model (ds, NULL);
+	gcups_driver_selector_set_nickname (ds, NULL);
 
-	if (use_detected) {
-		if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-			char *location;
-			LocalPrinter *desc;
-			gtk_tree_model_get (GTK_TREE_MODEL (list_store),
-					    &iter,
-					    1, &desc, -1);
-			location = g_strdup_printf (_("Local printer on %s"),
-						       desc->location);
-			gcups_driver_selector_set_vendor_and_model (ds, desc->vendor_and_model);
-			set_selected_uri (xml, desc->uri);
-			set_selected_location (xml, location);
-			g_free (location);
-		} else {
-			g_warning ("no selected printer");
-			return NULL;
-		}
-	} else {
+	if (!use_detected) {
 		LocalPrinter const *lp = local_port_get (xml);
 		if (NULL == lp)
 			return NULL;
 		set_selected_uri (xml, lp->uri);
-		set_selected_location (xml, lp->location);
+	} else if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		LocalPrinter *desc;
+		gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter,
+			1, &desc,
+			-1);
+		gcups_driver_selector_set_nickname (ds,
+			desc->vendor_and_model);
+		set_selected_uri (xml, desc->uri);
+	} else {
+		g_warning ("no selected printer");
+		return NULL;
 	}
 
 	return glade_xml_get_widget (xml, "details_page");
@@ -674,7 +659,6 @@ ui_auth_req_handler (GladeXML *xml)
 			_("Authentication Required"), msg->str, "", "", FALSE));
 		if (default_id != NULL)
 			gnome_password_dialog_set_username	(dialog, default_id);
-		gnome_password_dialog_set_show_userpass_buttons (dialog, FALSE);
 		gnome_password_dialog_set_show_username		(dialog,  TRUE);
 		gnome_password_dialog_set_show_domain		(dialog, FALSE);
 		gnome_password_dialog_set_show_password		(dialog,  TRUE);
@@ -922,6 +906,7 @@ get_manual_network_uri (GladeXML *xml)
 	return NULL;
 }
 
+#if 0
 static char *
 get_manual_network_location (GladeXML *xml)
 {
@@ -962,42 +947,7 @@ get_manual_network_location (GladeXML *xml)
 
 	return ret;
 }
-
-static char *
-lpd_get_vendor_and_model (char const *host)
-{
-	int retval;
-	char *ret = NULL;
-	char *detected = g_strdup (get_snmp_printers ((char*)host, &retval));
-
-	if (NULL != detected) {
-		ret = parse_network_detect (detected);
-		g_free (detected);
-	}
-	return ret;
-}
-
-static char *
-get_manual_network_vendor_and_model (GladeXML *xml)
-{
-	char *ret;
-	char *host;
-	host = NULL;
-
-	switch (get_manual_network_type (xml)) {
-	case NETWORK_TYPE_LPD :
-		host = entry_get_text_stripped (xml, "lpd_host_entry");
-		ret = lpd_get_vendor_and_model (host);
-		g_free (host);
-		return ret;
-	case NETWORK_TYPE_SMB :
-	case NETWORK_TYPE_IPP :
-		return NULL;
-	default :
-		g_warning ("unsupported type\n");
-		return NULL;
-	}
-}
+#endif
 
 static void
 cb_network_type_changed (GtkComboBox *combo, GladeXML *xml)
@@ -1043,23 +993,70 @@ network_location_back (GladeXML *xml)
 	return glade_xml_get_widget (xml, "local_or_remote_page");
 }
 
+static void
+cb_ipp_model (guint id, const char *path, ipp_t *response, GError **error,
+	      gpointer xml)
+{
+	ipp_attribute_t *attr;
+
+	if (!error && response)
+		for (attr = response->attrs; attr != NULL; attr = attr->next)
+			if (attr->name != NULL &&
+			    !g_ascii_strcasecmp (attr->name, "printer-make-and-model")) {
+				g_warning ("Found a %s", attr->values[0].string.text);
+				gcups_driver_selector_set_nickname (
+					driver_selector (xml), attr->values[0].string.text);
+			}
+
+	ippDelete (response);
+	g_clear_error (error);
+}
+
 static GtkWidget *
 network_location_next (GladeXML *xml)
 {
 	char *uri = get_manual_network_uri (xml);
 
 	if (uri != NULL) {
-		GCupsDriverSelector *ds = driver_selector (xml);
-		char *vendor_and_model = get_manual_network_vendor_and_model (xml);
-		char *location = get_manual_network_location (xml);
+		char *nickname = NULL;
+
+		switch (get_manual_network_type (xml)) {
+		case NETWORK_TYPE_IPP : {
+			static char const *attrs[] = {
+				  "printer-make-and-model"
+			};
+			ipp_t *request = gnome_cups_request_new_for_printer (
+				IPP_GET_PRINTER_ATTRIBUTES, uri);
+			gnome_cups_request_add_requested_attributes (request, 
+				IPP_TAG_OPERATION, G_N_ELEMENTS (attrs), (char**)attrs);
+			gnome_cups_request_execute_async (request, NULL, "/",
+				cb_ipp_model, g_object_ref (xml), g_object_unref);
+			break;
+		}
+
+		case NETWORK_TYPE_LPD : {
+			int snmp_err = 0;
+			char *host = entry_get_text_stripped (xml, "lpd_host_entry");
+			if (host != NULL) {
+				char const *detected = get_snmp_printers (host, &snmp_err);
+				if (NULL != detected && 0 == snmp_err)
+					nickname = parse_network_detect (detected);
+				g_free (host);
+			}
+			break;
+		}
+
+		default :
+			break;
+		}
+
+		gcups_driver_selector_set_nickname (
+			driver_selector (xml), nickname);
+		g_free (nickname);
 
 		set_selected_uri (xml, uri);
-		set_selected_location (xml, location);
 		g_free (uri);
-		g_free (location);
 
-		gcups_driver_selector_set_vendor_and_model (ds, vendor_and_model);
-		g_free (vendor_and_model);
 		return glade_xml_get_widget (xml, "details_page");
 	}
 	return NULL;
